/**
 * Copyright (c) 2011-2012, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

//----------------------------------------------------
//      Read temperature and humidity sensors, send to radio
//----------------------------------------------------

#include "stdfarmos.h"
#include <humidity.h>
#include <chips/ds18b20/ds18b20.h>
#include <chips/sht11/sht11_conv.h>
#include <adc.h>
#include <platform.h>
#include <net/simple_mhop.h>

struct Packet_s {
    uint16_t airTemp, airHum, soilHum;
    int16_t  soilTemp;
} PACKED;
typedef struct Packet_s Packet_t;

void handler(const void *buf, uint8_t len, addr_t sender)
{
    const Packet_t *p = buf;

    if (len != sizeof(Packet_t))
        return;

    PRINTF("Received raw sensor data: ");
    PRINTF("airTemp=%u airHum=%u ", p->airTemp, p->airHum);
    PRINTF("soilHum=%u soilTemp=%d\n", p->soilHum, p->soilTemp);
}

void appMain(void)
{
    PRINTF("Sensor read & send application\n");

    static Packet_t packet;

    netInit();
    netSetRecvHandler(handler);

#if !defined(USE_ROLE_BASE_STATION) && !defined(USE_ROLE_FORWARDER)
    mdelay(100); // Wait for sensors to initialize

    humidityOn();
    adcSetChannel(ADC_SOIL_HUMIDITY);
#endif

    while (1) {
#if !defined(USE_ROLE_BASE_STATION) && !defined(USE_ROLE_FORWARDER)
        packet.airTemp  = temperatureRead();
        packet.airHum   = humidityRead();
        packet.soilTemp = ds18b20Measure();
        packet.soilHum  = adcReadFast();

        netSend(&packet, sizeof(packet));
#endif

        mdelay(3000);
    }
}
