/**
 * Copyright (c) 2011, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdfarmos.h>

/*

Calculating internal temperature:
=================================

V_sensor,typ = TC_sensor (273 + T[degrees Celsium]) + V_offset,sensor[mV]
 or
V_sensor,typ = TC_sensor T[Celsium degrees] + V_sensor(T_A = 0 degrees Celsium)[mV]

Assuming V_offset == 0 mV and TC_sensor == 3.55 mV/degree:
     T  = (V - 3.55 * 273) / 3.55 ~=
       ~= (V * 20 - 19383) / 71

For better precision the sensor must be calibarted!


Calculating internal voltage:
=============================

V = V_sensor * 3 / 1024

*/

void appMain(void)
{
    for (;;) {
        uint16_t value;
        uint16_t temperature;
        uint16_t volts;

        value = adcRead(ADC_INTERNAL_TEMPERATURE);
        temperature = (long)value * 705 / 1024 - 278;
        PRINTF("temp: %u (%u C)", value, temperature);

        value = adcRead(ADC_INTERNAL_VOLTAGE);
        volts = 100 * (long)value / 207;
        PRINTF(" volts: %u (%u.%u V)\n", value, volts / 100, volts % 100);

        ledToggle();
        mdelay(1000);
    }
}
