/**
 * Copyright (c) 2011, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdfarmos.h>
#include <string.h>

void sendCounter(void);
void recvCounter(void);
void ccaTest(void);

uint8_t buffer[RADIO_MAX_PACKET];

//-------------------------------------------
//      Entry point for the application
//-------------------------------------------
void appMain(void)
{
    // ------------- setup radio
    // -- parameters
    // radioSetTxPower(30);
    // -- set packet rx callback
    radioSetReceiveHandle(recvCounter);
    // -- turn listening on
    radioOn();
    // -------------

    //sendCounter(); // Uncomment this for the sending node

    for (;;) {
        mdelay(500);
        ledToggle();
    }
}

void recvCounter(void)
{
    int16_t len;
    uint8_t *counter = &buffer[0];

    len = radioRecv(buffer, sizeof(buffer));
    if (len < 0) {
        PRINTF("radio receive failed\n");
        return;
    }
    if (len == 0) {
        PRINTF("received zero length!\n");
    } else {
        PRINTF("received counter %u, len %u, rssi=%d, lqi=%u\n",
                *counter, len, radioGetLastRSSI(), radioGetLastLQI());
        debugHexdump(buffer, 12);
    }
}

void sendCounter(void)
{
    uint8_t counter = 0;
    for (;;) {
        PRINTF("sending counter %i\n", counter);
        ledToggle();
        buffer[0] = counter;
        radioSend(buffer, sizeof(buffer));
        mdelay(1000);
        counter++;
    }
}
