/**
 * Copyright (c) 2011, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdfarmos.h>
#include "../common.h"

#define MAX_NEIGHBORS 11

uint8_t nbrCount;
RadioInfoPacket_t nbrs[MAX_NEIGHBORS];

bool checkNeighbor(uint16_t addr, RadioInfoPacket_t *packet)
{
    uint8_t i;
    for (i = 0; i < nbrCount; i++) {
        if (nbrs[i].address == packet->address) {
            nbrs[i] = *packet;
            return true;
        }
    }

    // If we got so far it means that there's no such neighbor, let's add one
    if (nbrCount < MAX_NEIGHBORS) {
        nbrs[nbrCount++] = *packet;
        return true;
    } 
    return false;
}

void recvCallback(void)
{
    RadioInfoPacket_t packet;
    int16_t len;

    greenLedToggle();
    len = radioRecv(&packet, sizeof(packet));
    if (len < 0) {
        PRINT("radio receive failed\n");
        return;
    }
    if (len < sizeof(packet)) {
        PRINT("too small!\n");
        return;
    }
    // PRINT("radio rx!\n");
    checkNeighbor(packet.address, &packet);
}

void dumpNeighbors(void)
{
    uint8_t i;
    for (i = 0; i < nbrCount; i++) {
        RadioInfoPacket_t *p = &nbrs[i];
        PRINTF("%04x: last: %d num: %d PDR: %d%% ne: %d PDR-NE: %d%% rssi: %d lqi: %d\n",
                p->address, p->lastTestNo, p->numTests, p->avgPdr,
                p->numTestsNe, p->avgPdrNe, p->avgRssiNe, p->avgLqiNe);
    }
}

void appMain(void)
{
    radioSetChannel(BS_CHANNEL);
    radioSetReceiveHandle(recvCallback);
    radioOn();

    for (;;) {
        PRINT(".\n");
        DISABLE_INTS();
        dumpNeighbors();
        ENABLE_INTS();
        mdelay(5000);
    }
}
