/**
 * Copyright (c) 2011, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdfarmos.h"
#include <vlit.h>
#include <net/simple_mhop.h>
#include <string.h>

#define SIMULATION_MODE		1

uint16_t airTemp, airHum, soilHum, soilTemp;
struct vlit_values_t data;

    // send to the gateway
    /*static vlit_data_t data2;
    data2.metadata.rf_adr = src;
    data2.metadata.app_len = sizeof(vlit_values_t);
    data2.metadata.app_cmd = 0;

    data2.values.soil_temp = nodeData[i].soil_temp;
    data2.values.soil_humi = nodeData[i].soil_humi;
    data2.values.air_temp = nodeData[i].air_temp;
    data2.values.air_humi = nodeData[i].air_humi;
    data2.values.bat_volt = nodeData[i].bat_volt;
    data2.values.rssi = nodeData[i].rssi;

    vlitSend(UMC_CMD_RECEIVE_DATA, &data2, sizeof(data2), src);*/

// Handler for packets received by base station
void handler(const void *buf, uint8_t len, addr_t sender)
{
    // read the sensors
    memcpy(&soilTemp, (const char*)buf, 2);
    memcpy(&soilHum, (const char*)buf+2, 2);
    memcpy(&airTemp, (const char*)buf+4, 2);
    memcpy(&airHum, (const char*)buf+6, 2);

    data.soil_temp = soilTemp;
    data.soil_humi = soilHum;
    data.air_temp = airTemp;
    data.air_humi = airHum;
    data.bat_volt = 0;
    data.rssi = 0;
    setNodeData(&data, sender);
}

void appMain(void)
{
    //PRINTF("App is started\n");

    // Initialize the network
    netInit();
    // Initialize VLIT uart channel
    vlitInit();
    // Set received packet handler -- for base station
    netSetRecvHandler(handler);

    if (SIMULATION_MODE) {
        uint8_t buf[sizeof(uint8_t) + (sizeof(uint8_t) + sizeof(uint32_t)) * 11];
        uint8_t offset = 0;
        uint8_t numValues = 11;
        uint8_t type = 16;
        uint32_t value = 1;
        uint8_t i;

        memcpy(buf, &numValues, sizeof(uint8_t));
        offset += sizeof(uint8_t);
        for (i = 0; i < numValues; i++) {
            memcpy(buf + offset, &type, sizeof(uint8_t));
            offset += sizeof(uint8_t);
            memcpy(buf + offset, &value, sizeof(uint32_t));
            offset += sizeof(uint32_t);
            type++;
            value++;
        }
        while (1) {
            mdelay(7000);
            vlitSend(UMC_CMD_RECEIVE_DATA_ARB, buf, offset, 42);
        }
    } else {
        while (1) {
	    mdelay(10000);
        }
    }
}
