/*
 * Copyright (c) 2011-2012, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <net/address.h>
#include <msp430/msp430_flash.h> // TODO: use hil interface
#include <lib/hash.h>
#include <print.h>
#include <assert.h>
#include <serial_number.h>

#define LOCAL_ADDR_STORAGE_ADDRESS (MSP430_FLASH_INFOMEM_START + MSP430_FLASH_INFOMEM_SEGMENT_SIZE)

// generate local address from serial number
static inline uint16_t generateLocalAddress(void) 
{
#if PREDEFINED_LOCAL_ADDRESS && !CHECK_PREDEFINED_ADDRESS
    return PREDEFINED_LOCAL_ADDRESS;
#endif

    uint16_t snum[4];
    bool ok = serialNumberRead((uint8_t *) snum);
    (void)ok;
    // PRINTF("generateLocalAddress... ok=%s\n", (ok ? "yes" : "no"));

    uint16_t result = 0;
    int i;
    for (i = 0; i < 4; ++i) {
        // PRINTF("snum[%d]=%#04x\n", i, snum[i]);
        result += snum[i];
    }

    // do not allow the result to be one of reserved addresses
    if (result == MOS_ADDR_ROOT
            || result == MOS_ADDR_BASESTATION) {
        result = 0x2;
    }

    // set highest bit to zero
    return result & ~0x8000;
}

static inline void setupLocalAddr(void)
{
    if (localAddress != 0 && localAddress != 0xffff) {
        // PRINTF("setupLocalAddr: address ok: %d\n", localAddress);
        return;
    }
#if USE_FLASH
    msp430flashReadWord(LOCAL_ADDR_STORAGE_ADDRESS, &localAddress);
    if (localAddress == 0 || localAddress == 0xffff) { 
        localAddress = generateLocalAddress();
        msp430flashErase(LOCAL_ADDR_STORAGE_ADDRESS, sizeof(uint16_t));
        msp430flashWriteWord(LOCAL_ADDR_STORAGE_ADDRESS, localAddress);
        // PRINTF("setupLocalAddr: written in flash address %d\n", localAddress);
    }
#else
    localAddress = generateLocalAddress();
#endif

#if PREDEFINED_LOCAL_ADDRESS && CHECK_PREDEFINED_ADDRESS
    if (PREDEFINED_LOCAL_ADDRESS != localAddress) {
        ASSERT(!"local address mismatch");
    }
#endif

}

void networkingInitArch(void)
{
#if USE_ROLE_BASE_STATION
    localAddress = MOS_ADDR_BASESTATION;
#else
    setupLocalAddr();
#endif
}
