/*
 * Copyright (c) 2011-2013, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "windspeed.h"
#include <print.h>

//
// Wind speed sensor.
// Generates periodic interrupts depending on the speed of wind.
// This driver returns the number of ticks in last 10 seconds, updated once per 2 seconds.
//

static uint8_t windspeedCounter;
static uint8_t lastWindspeedCounter[5];

void windspeedInit(void)
{
    pinClearIntFlag(PLATFORM_WINDSPEED_PORT, PLATFORM_WINDSPEED_PIN);
    pinIntRising(PLATFORM_WINDSPEED_PORT, PLATFORM_WINDSPEED_PIN);
    pinEnableInt(PLATFORM_WINDSPEED_PORT, PLATFORM_WINDSPEED_PIN);

    // Configure Timer B
    TBCTL = TBCLR;

    // start Timer B
    TBCTL = TBSSEL_1 + MC_CONT + TBIE; // clock = ACLK, continuos, interrupt
}

// return readings from the previous period
uint16_t windspeedRead(void)
{
    return lastWindspeedCounter[0] + lastWindspeedCounter[1]
           + lastWindspeedCounter[2] + lastWindspeedCounter[3] + lastWindspeedCounter[4];
}

// hand
void windspeedInterruptHandle(void)
{
    windspeedCounter++;
}


ISR(TIMERB1, windspeedTimerInterrupt)
{
//    PRINTF("timer B interrupt\n");

    // overflow?
    if (TBIV != 14) return;

    static uint8_t pos;
    // save old count
    lastWindspeedCounter[pos++] = windspeedCounter;
    // reset the counter
    windspeedCounter = 0;
    // reset the ticker
    if (pos == 5) pos = 0;
}
