/*
 * Copyright (c) 2011, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * radio_hal_cc1101.h -- CC1101 radio driver glue
 */

#ifndef PLATFORM_RADIO_H
#define PLATFORM_RADIO_H

#include <cc1101/cc1101.h>
#include <string.h>

#define RADIO_MAX_PACKET    CC1101_MAX_PACKET_LEN
#define RADIO_TX_POWER_MIN  CC1101_TX_POWER_MIN
#define RADIO_TX_POWER_MAX  CC1101_TX_POWER_MAX

static unsigned char key1[]   = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                                 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};
static unsigned char key2[]   = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                                 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};

static inline void radioInit(void)
{
    cc1101Init();
}

static inline void radioReinit(void)
{
    cc1101InitSpi();
}

static inline int8_t radioSendHeader(const void *header, uint16_t headerLength,
                                     const void *data,   uint16_t dataLength)
{
    unsigned char buf[128];
    uint16_t len = (headerLength + dataLength);
    uint16_t offset = sizeof(uint16_t);
    uint16_t offset2 = offset + headerLength;
    unsigned char key[16];
    
    memset(buf, 0, sizeof(buf));
    memcpy(buf, &len, sizeof(uint16_t));
    if (headerLength > 0) {
        memcpy(buf + offset, header, headerLength);
        offset += headerLength;
    }
    if (dataLength > 0) {
        memcpy(buf + offset, data, dataLength);
    }
    offset = len + sizeof(uint16_t);
#if USE_CRYPTO
    offset = 0;
    while (offset < len + sizeof(uint16_t)) {
        memcpy(key, key1, 16);
        aes_enc_dec(buf + offset, key, 0);
        offset += 16;
    }
#endif
    return cc1101Send(buf, offset2, buf + offset2, offset - offset2);
}

static inline int16_t radioRecv(void *buffer, uint16_t bufferLength)
{
    int16_t len = cc1101Read(buffer, bufferLength);
    if (len > 0) {
        unsigned char buf[128];
        unsigned char key[16];
        uint16_t offset = 0;         

	memset(buf, 0, sizeof(buf));
        memcpy(buf, buffer, len);
#if USE_CRYPTO
        while (offset < len) {
            memcpy(key, key2, 16);
            aes_enc_dec(buf + offset, key, 1);
            offset += 16;
        }
#endif
        memcpy(&len, buf, sizeof(uint16_t));
        memcpy(buffer, buf + sizeof(uint16_t), len);
    }
    return len;
}

static inline void radioDiscard(void)
{
    cc1101Discard();
}

static inline RadioRecvFunction radioSetReceiveHandle(
    RadioRecvFunction functionHandle)
{
    cc1101SetRecvCallback(functionHandle);
    return NULL;
}

static inline void radioOn(void)
{
    cc1101On();
}

static inline void radioOff(void)
{
    cc1101Off();
}

static inline int radioGetRSSI(void)
{
    return cc1101GetRSSI();
}

static inline int8_t radioGetLastRSSI(void)
{
    return cc1101GetLastRSSI();
}

static inline uint8_t radioGetLastLQI(void)
{
    return cc1101GetLastLQI();
}

static inline void radioSetChannel(int channel)
{
    cc1101SetChannel(channel);
}

static inline void radioSetTxPower(uint8_t power)
{
    cc1101SetTxPower(power);
}

static inline bool radioIsChannelClear(void)
{
    return cc1101IsChannelClear();
}

#endif
